/* If the timestep changes by a factor "base" and the
 * errors are e1 and e2, respectively, then the order
 * is log(e1/e2)/log(base)
 */

#include "misc.h"
#include "variable_leapfrog.h"
#include "lsode.h"
#include "matvec.h"
#include <assert.h>
#include <math.h>

#define NUM_H 20

/* Set T_FINAL to DT to measure local error, or a constant to measure global */
#define T_FINAL 1.0

#if 0
/* Pendulum */
#define PENDULUM 1
#define N 2
#define F pendulum
#define F2 pendulum2
void pendulum(int n, double t, double *y, double *yp)
{
    assert(n==2);
    yp[0] = y[1];
    yp[1] = -y[0];
}

void pendulum2(int n, double t, double *r, double *rpp)
{
    assert(n==1);
    rpp[0] = -r[0];
}
#else
/* Kepler problem */
#define KEPLER 1
#define N 4
#define F Kepler
#define F2 Kepler2
void Kepler(int n, double t, double y[n], double yp[n])
{
    double r;
    assert(n==4);
    yp[0]=y[2];
    yp[1]=y[3];
    r=sqrt(SQR(y[0])+SQR(y[1]));
    yp[2]=-y[0]/(r*r*r);
    yp[3]=-y[1]/(r*r*r);
}
void Kepler2(int n, double t, double *y, double *ypp)
{
    double r;
    assert(n==2);
    r=sqrt(SQR(y[0])+SQR(y[1]));
    ypp[0]=-y[0]/(r*r*r);
    ypp[1]=-y[1]/(r*r*r);
//    printf("%g %g\n", y[0], y[1]);
}
#endif

static double DT;

double ComputeDT(int n, double T, double current_DT, double *R, double *V)
{
#if 0
    static int half = 0;
    extern double drand48(void);
    if(drand48() < 0.00)
	half = !half;
    if(half)
	return DT/2;
    else
#endif
	return DT;
}

int main(void)
{
    double base=2.0, ytrue[N], yleap[N], error[NUM_H];
    int i;
    LSODE *lsode;
    
    srand48(time(NULL)+getpid());

    for(i=0; i<NUM_H; i++)
    {
	double diff[N];

	DT = IntPow(base, -i);

#if PENDULUM == 1
	ytrue[0] = 1.0;
	ytrue[1] = 0.0;
#elif KEPLER == 1
	ytrue[0] = 0.0;
	ytrue[1] = 1.0;
	ytrue[2] = 1.0;
	ytrue[3] = 0.0;
#endif
//	printf("N=%d", N);
//	for(i=0; i<N;i++) printf(" %g", ytrue[i]);
//	printf("\n");
	VecCopy(N, yleap, ytrue);

	lsode = LsodeAlloc(N, 0.0, ytrue, F, 0, 4e-16, 0.0);
	assert(T_FINAL ==  LsodeIntegrate(lsode, T_FINAL));
	LsodeFree(lsode);

//	printf("INIT Leap: %g %g %g %g\n", yleap[0], yleap[1], yleap[2], yleap[3]);
	init_variable_leapfrog(N/2, 0.0, ComputeDT, yleap, yleap+N/2, F2);
	assert(T_FINAL == integrate_variable_leapfrog(T_FINAL));

	VecDiff(N, diff, ytrue, yleap);
	error[i] = VecNorm1(N, diff);
	if(i > 0)
	{
	    double order = log(error[i-1]/error[i])/log(base);
	    printf("%2d %g %g %g\n", i, DT, error[i], order);
	}
    }

    return 0;
}
